#!/bin/bash

irw > /tmp/remote_command &

CCONTROL_PID=$!

# BABOOSHKA_HOME=/home/pi	# assigned in start.babooshka.sh

DBUS_CTRL_FILE=/tmp/dbuscontrol_command
SAVEPATH=$BABOOSHKA_HOME

# --------------------------------------

killCCONTROL() {	# function called by trap (Ctrl-C handler)

	echo CCONTROL_PID $CCONTROL_PID

	kill -9 $CCONTROL_PID 2> /dev/null
	echo "Have a nice day..."
	exit 0
}

trap 'killCCONTROL' 2

# --------------------------------------

beep (){
	play -n synth sin %41 fade q 0.0 0.1 0.01 gain -25 &>/dev/null </dev/null &
}

prevData=0
musicPlay=0
pressCnt=0

while [ 1 ]; do

	data=$(stdbuf -i0 -o0 -e0 tail -1 /tmp/remote_command | awk -F' ' '{print $3; exit}' | xargs )

	pressCnt=$(($pressCnt + 1))

	# ---- note repeatable -----

	if [ "$prevData" != "$data" ]; then
	
		case "$data" in
			"qqqq")
				$BABOOSHKA_HOME/dbuscontrol.sh stop
				echo qqqq
			;;
			"wwww")
				echo wwww
			;;
		esac

		prevData=$data
	fi 

	# ---- repeatable ---------

	case "$data" in

			# next

			"ARR_DOWN")
				beep
				echo "NEXT" > "$DBUS_CTRL_FILE"
				$BABOOSHKA_HOME/dbuscontrol.sh stop
				echo "NEXT"
			;;

			# prev

			"ARR_UP")
				beep
				echo "PREV" > "$DBUS_CTRL_FILE"
				$BABOOSHKA_HOME/dbuscontrol.sh stop
				echo "PREV"
			;;

			# forward +10min

			"ARR_RIGHT")
				beep
				echo "NEXT" > "$DBUS_CTRL_FILE"
				$BABOOSHKA_HOME/dbuscontrol.sh seekfl
				echo ">>"
			;;

			# backward -10min

			"ARR_LEFT")
				beep
				$BABOOSHKA_HOME/dbuscontrol.sh seekbl
				echo "<<"
			;;

			# play/pause toggle

			"ARR_OK")
				beep
				status=$(./dbuscontrol.sh status 2>/dev/null)

				if [ "$status" != "" ]; then
					position=$( echo "$status" | grep Position | awk '{split($0,a,":"); print a[2]}' )
					position=$(( $position / (1000000) ))	# sec
					echo $position > $SAVEPATH/currentpos
					paused=$( echo "$status" | grep Paused | awk '{split($0,a,":"); print a[2]}' | xargs )
				fi

				if [ "$paused" == "true" ] || [ "$status" == "" ]; then # has been paused 
					echo "PLAY"
					if [ "$status" == "" ]; then echo "PLAY" > "$DBUS_CTRL_FILE"; fi
				else 
					echo "PAUSE"
				fi
				$BABOOSHKA_HOME/dbuscontrol.sh pause
			;;


			# pause on switching-off

			"EXT"|"ON_OFF")	# AV or ON_OFF
				beep

				status=$(./dbuscontrol.sh status 2>/dev/null)

				if [ "$status" != "" ]; then
					position=$( echo "$status" | grep Position | awk '{split($0,a,":"); print a[2]}' )
					position=$(( $position / (1000000) ))	# sec
					echo $position > $SAVEPATH/currentpos
					paused=$( echo "$status" | grep Paused | awk '{split($0,a,":"); print a[2]}' | xargs )
				fi

#				paused=$( echo "$status" | grep Paused | awk '{split($0,a,":"); print a[2]}' | xargs )

				if [ "$paused" == "false" ]; then # not paused 
					$BABOOSHKA_HOME/dbuscontrol.sh pause
#					echo "$status" > status
					echo "PAUSED"
				fi
				echo "AV or ON-OFF"
			;;

			# stop

			"KEY_TEXT")
				beep

				status=$(./dbuscontrol.sh status 2>/dev/null)

				if [ "$status" != "" ]; then
					position=$(echo "$status" | grep Position | awk '{split($0,a,":"); print a[2]}' )
					position=$(( $position / (1000000) ))	# sec
					echo $position > $SAVEPATH/currentpos

					echo "STOP" > "$DBUS_CTRL_FILE"
					$BABOOSHKA_HOME/dbuscontrol.sh stop
					echo "STOP"
				else
					echo "STOP / NOT PLAYING"
				fi

			;;

			# audio track switch		

			"KEY_RED")
				audioTracks=$( $BABOOSHKA_HOME/dbuscontrol.sh audiolist 2>/dev/null ) 
				trackCnt=$( echo "$audioTracks" | sed '/^\s*$/d' | wc -l )
				currentTrack=$( echo "$audioTracks" | awk '/active/{ print NR; exit }' )
				beep
				if [ "$trackCnt" != "0" ]; then
					if [ "$currentTrack" -lt "$trackCnt" ]; then 
						$BABOOSHKA_HOME/dbuscontrol.sh audiop
						echo "AUDIO CHANGE" 
					else 
						for var in $audioTracks; do
							$BABOOSHKA_HOME/dbuscontrol.sh audiom
						done 
						echo "AUDIO REVIND"
					fi 
				fi
				echo "AUDIO: $trackCnt, $currentTrack"
			;;

			# subtitles switch		

			"KEY_YELLOW")
				subsTracks=$( $BABOOSHKA_HOME/dbuscontrol.sh subslist 2>/dev/null ) 
				subsCnt=$( echo "$subsTracks" | sed '/^\s*$/d' | wc -l )
				currentSub=$( echo "$subsTracks" | awk '/active/{ print NR; exit }' )
				beep
				if [ "$subsCnt" != "0" ]; then
					if [ "$currentSub" -lt "$subsCnt" ]; then 
						$BABOOSHKA_HOME/dbuscontrol.sh subsnext
						echo "SUBS CHANGE" 
					else 
						if [ "$subsOff" != "1" ]; then 
							$BABOOSHKA_HOME/dbuscontrol.sh hidesubtitles
							subsOff=1
							echo "SUBS HIDE"
						else  
							$BABOOSHKA_HOME/dbuscontrol.sh showsubtitles
							for var in $subsTracks; do
								$BABOOSHKA_HOME/dbuscontrol.sh subsprev
							done 
							subsOff=0
							echo "SUBS REVIND"
						fi
					fi 
				fi
				echo "SUBS: $subsCnt, $currentSub"
			;;

			*)
#				echo ------
			;;
	esac

	: > /tmp/remote_command

	sleep 0.5
done





